@@ -2,9 +2,12 @@ module RSpec
   module Mocks
     module AnyInstance
       class Chain
-        class << self
-          private
+        def initialize(*args, &block)
+          @expectation_args  = args
+          @expectation_block = block
+        end
 
+        module Customizations
           # @macro [attach] record
           #   @method $1(*args, &block)
           #   Records the `$1` message for playback against an instance that
@@ -12,34 +15,37 @@ module RSpec
           #
           #   @see RSpec::Mocks::MessageExpectation#$1
           #
-          def record(method_name)
+          def self.record(method_name)
             class_eval(<<-EOM, __FILE__, __LINE__ + 1)
               def #{method_name}(*args, &block)
                 record(:#{method_name}, *args, &block)
               end
             EOM
           end
+
+          record :and_return
+          record :and_raise
+          record :and_throw
+          record :and_yield
+          record :and_call_original
+          record :with
+          record :once
+          record :twice
+          record :any_number_of_times
+          record :exactly
+          record :times
+          record :never
+          record :at_least
+          record :at_most
         end
 
-        record :and_return
-        record :and_raise
-        record :and_throw
-        record :and_yield
-        record :and_call_original
-        record :with
-        record :once
-        record :twice
-        record :any_number_of_times
-        record :exactly
-        record :times
-        record :never
-        record :at_least
-        record :at_most
+        include Customizations
 
         # @private
         def playback!(instance)
-          messages.inject(instance) do |_instance, message|
-            _instance.__send__(*message.first, &message.last)
+          message_expectation = create_message_expectation_on(instance)
+          messages.inject(message_expectation) do |object, message|
+            object.__send__(*message.first, &message.last)
           end
         end
 
@@ -57,8 +63,17 @@ module RSpec
           @expectation_fulfilled = true
         end
 
+        def never
+          ErrorGenerator.raise_double_negation_error("expect_any_instance_of(MyClass)") if negated?
+          super
+        end
+
         private
 
+        def negated?
+          messages.any? { |(message, *_), _| message.to_sym == :never }
+        end
+
         def messages
           @messages ||= []
         end
