@@ -60,8 +60,7 @@ module RSpec
         end
 
         def should_not_receive(method_name, &block)
-          observe!(method_name)
-          message_chains.add(method_name, NegativeExpectationChain.new(method_name, &block))
+          should_receive(method_name, &block).never
         end
 
         # Removes any previously recorded stubs, stub_chains or message
@@ -84,6 +83,9 @@ module RSpec
           if @expectation_set && !message_chains.all_expectations_fulfilled?
             raise RSpec::Mocks::MockExpectationError, "Exactly one instance should have received the following message(s) but didn't: #{message_chains.unfulfilled_expectations.sort.join(', ')}"
           end
+        ensure
+          stop_all_observation!
+          ::RSpec::Mocks.space.remove_any_instance_recorder_for(@klass)
         end
 
         # @private
@@ -103,7 +105,7 @@ module RSpec
 
         # @private
         def playback!(instance, method_name)
-          RSpec::Mocks::space.add(instance)
+          RSpec::Mocks.space.ensure_registered(instance)
           message_chains.playback!(instance, method_name)
           @played_methods[method_name] = instance
           received_expected_message!(method_name) if message_chains.has_expectation?(method_name)
@@ -181,7 +183,7 @@ module RSpec
           @klass.class_eval(<<-EOM, __FILE__, __LINE__ + 1)
             def #{method_name}(*args, &blk)
               klass = ::RSpec::Mocks.method_handle_for(self, :#{method_name}).owner
-              klass.__recorder.playback!(self, :#{method_name})
+              ::RSpec::Mocks.any_instance_recorder_for(klass).playback!(self, :#{method_name})
               self.__send__(:#{method_name}, *args, &blk)
             end
           EOM
@@ -193,7 +195,7 @@ module RSpec
             def #{method_name}(*args, &blk)
               method_name = :#{method_name}
               klass = ::RSpec::Mocks.method_handle_for(self, :#{method_name}).owner
-              invoked_instance = klass.__recorder.instance_that_received(method_name)
+              invoked_instance = ::RSpec::Mocks.any_instance_recorder_for(klass).instance_that_received(method_name)
               raise RSpec::Mocks::MockExpectationError, "The message '#{method_name}' was received by \#{self.inspect} but has already been received by \#{invoked_instance}"
             end
           EOM
