@@ -3,51 +3,35 @@ module RSpec
 
     class MessageExpectation
       # @private
+      attr_accessor :error_generator, :implementation
       attr_reader :message
-      attr_writer :expected_received_count, :expected_from, :argument_expectation, :implementation
-      protected :expected_received_count=, :expected_from=, :implementation=
-      attr_accessor :error_generator
-      protected :error_generator, :error_generator=
+      attr_writer :expected_received_count, :expected_from, :argument_list_matcher
+      protected :expected_received_count=, :expected_from=, :error_generator, :error_generator=, :implementation=
 
       # @private
-      def initialize(error_generator, expectation_ordering, expected_from, message, expected_received_count=1, opts={}, &implementation)
+      def initialize(error_generator, expectation_ordering, expected_from, method_double,
+                     expected_received_count=1, opts={}, &implementation_block)
         @error_generator = error_generator
         @error_generator.opts = opts
         @expected_from = expected_from
-        @message = message
+        @method_double = method_double
+        @message = @method_double.method_name
         @actual_received_count = 0
         @expected_received_count = expected_received_count
-        @argument_expectation = ArgumentExpectation.new(ArgumentMatchers::AnyArgsMatcher.new)
-        @consecutive = false
-        @exception_to_raise = nil
-        @args_to_throw = []
+        @argument_list_matcher = ArgumentListMatcher.new(ArgumentMatchers::AnyArgsMatcher.new)
         @order_group = expectation_ordering
         @at_least = @at_most = @exactly = nil
         @args_to_yield = []
         @failed_fast = nil
-        @args_to_yield_were_cloned = false
         @eval_context = nil
-        @implementation = implementation
-      end
 
-      # @private
-      def build_child(expected_from, expected_received_count, opts={}, &implementation)
-        child = clone
-        child.expected_from = expected_from
-        child.implementation = implementation if implementation
-        child.expected_received_count = expected_received_count
-        child.clear_actual_received_count!
-        new_gen = error_generator.clone
-        new_gen.opts = opts
-        child.error_generator = new_gen
-        child.clone_args_to_yield(*@args_to_yield)
-        child.argument_expectation = ArgumentExpectation.new(ArgumentMatchers::AnyArgsMatcher.new)
-        child
+        @implementation = Implementation.new
+        self.inner_implementation_action = implementation_block
       end
 
       # @private
       def expected_args
-        @argument_expectation.args
+        @argument_list_matcher.expected_args
       end
 
       # @overload and_return(value)
@@ -87,13 +71,44 @@ module RSpec
       #   counter.stub(:count) { 1 }
       #   counter.count # => 1
       def and_return(*values, &implementation)
+        if negative?
+          RSpec.deprecate "`and_return` on a negative message expectation"
+        end
+
         @expected_received_count = [@expected_received_count, values.size].max unless ignoring_args? || (@expected_received_count == 0 and @at_least)
-        @consecutive = true if values.size > 1
-        @implementation = implementation || build_implementation(values)
+
+        if implementation
+          # TODO: deprecate `and_return { value }`
+          self.inner_implementation_action = implementation
+        else
+          self.terminal_implementation_action = AndReturnImplementation.new(values)
+        end
+
+        nil
+      end
+
+      # Tells the object to delegate to the original unmodified method
+      # when it receives the message.
+      #
+      # @note This is only available on partial mock objects.
+      #
+      # @example
+      #
+      #   counter.should_receive(:increment).and_call_original
+      #   original_count = counter.count
+      #   counter.increment
+      #   expect(counter.count).to eq(original_count + 1)
+      def and_call_original
+        if @method_double.object.is_a?(RSpec::Mocks::TestDouble)
+          @error_generator.raise_only_valid_on_a_partial_mock(:and_call_original)
+        else
+          @implementation = AndCallOriginalImplementation.new(@method_double.original_method)
+        end
       end
 
       # @overload and_raise
       # @overload and_raise(ExceptionClass)
+      # @overload and_raise(ExceptionClass, message)
       # @overload and_raise(exception_instance)
       #
       # Tells the object to raise an exception when the message is received.
@@ -101,17 +116,24 @@ module RSpec
       # @note
       #
       #   When you pass an exception class, the MessageExpectation will raise
-      #   an instance of it, creating it with `new`. If the exception class
-      #   initializer requires any parameters, you must pass in an instance and
-      #   not the class.
+      #   an instance of it, creating it with `exception` and passing `message`
+      #   if specified.  If the exception class initializer requires more than
+      #   one parameters, you must pass in an instance and not the class,
+      #   otherwise this method will raise an ArgumentError exception.
       #
       # @example
       #
       #   car.stub(:go).and_raise
       #   car.stub(:go).and_raise(OutOfGas)
+      #   car.stub(:go).and_raise(OutOfGas, "At least 2 oz of gas needed to drive")
       #   car.stub(:go).and_raise(OutOfGas.new(2, :oz))
-      def and_raise(exception=Exception)
-        @exception_to_raise = exception
+      def and_raise(exception = RuntimeError, message = nil)
+        if exception.respond_to?(:exception)
+          exception = message ? exception.exception(message) : exception.exception
+        end
+
+        self.terminal_implementation_action = Proc.new { raise exception }
+        nil
       end
 
       # @overload and_throw(symbol)
@@ -124,8 +146,9 @@ module RSpec
       #
       #   car.stub(:go).and_throw(:out_of_gas)
       #   car.stub(:go).and_throw(:out_of_gas, :level => 0.1)
-      def and_throw(symbol, object = nil)
-        @args_to_throw = [symbol, object].compact
+      def and_throw(*args)
+        self.terminal_implementation_action = Proc.new { throw(*args) }
+        nil
       end
 
       # Tells the object to yield one or more args to a block when the message
@@ -135,44 +158,34 @@ module RSpec
       #
       #   stream.stub(:open).and_yield(StringIO.new)
       def and_yield(*args, &block)
-        if @args_to_yield_were_cloned
-          @args_to_yield.clear
-          @args_to_yield_were_cloned = false
-        end
-
         yield @eval_context = Object.new.extend(RSpec::Mocks::InstanceExec) if block
-
         @args_to_yield << args
+        self.initial_implementation_action = AndYieldImplementation.new(@args_to_yield, @eval_context, @error_generator)
         self
       end
 
       # @private
       def matches?(message, *args)
-        @message == message && @argument_expectation.args_match?(*args)
+        @message == message && @argument_list_matcher.args_match?(*args)
       end
 
       # @private
-      def invoke(*args, &block)
-        if (@expected_received_count == 0 && !@at_least) || ((@exactly || @at_most) && (@actual_received_count == @expected_received_count))
+      def invoke(parent_stub, *args, &block)
+        if negative? || ((@exactly || @at_most) && (@actual_received_count == @expected_received_count))
           @actual_received_count += 1
           @failed_fast = true
-          @error_generator.raise_expectation_error(@message, @expected_received_count, @actual_received_count, *args)
+          #args are the args we actually received, @argument_list_matcher is the
+          #list of args we were expecting
+          @error_generator.raise_expectation_error(@message, @expected_received_count, @argument_list_matcher, @actual_received_count, expectation_count_type, *args)
         end
 
         @order_group.handle_order_constraint self
 
         begin
-          raise_exception unless @exception_to_raise.nil?
-          Kernel::throw(*@args_to_throw) unless @args_to_throw.empty?
-
-          default_return_val = call_with_yield(&block) if !@args_to_yield.empty? || @eval_context
-
-          if @consecutive
-            call_implementation_consecutive(*args, &block)
-          elsif @implementation
-            call_implementation(*args, &block)
-          else
-            default_return_val
+          if implementation.present?
+            implementation.call(*args, &block)
+          elsif parent_stub
+            parent_stub.invoke(nil, *args, &block)
           end
         ensure
           @actual_received_count += 1
@@ -180,26 +193,21 @@ module RSpec
       end
 
       # @private
-      def raise_exception
-        if !@exception_to_raise.respond_to?(:instance_method) ||
-            @exception_to_raise.instance_method(:initialize).arity <= 0
-          raise(@exception_to_raise)
-        else
-          raise ArgumentError.new(<<-MESSAGE)
-'and_raise' can only accept an Exception class if an instance can be constructed with no arguments.
-#{@exception_to_raise.to_s}'s initialize method requires #{@exception_to_raise.instance_method(:initialize).arity} arguments, so you have to supply an instance instead.
-MESSAGE
-        end
+      def negative?
+        @expected_received_count == 0 && !@at_least
       end
 
       # @private
       def called_max_times?
-        @expected_received_count != :any && @expected_received_count > 0 && @actual_received_count >= @expected_received_count
+        @expected_received_count != :any &&
+          !@at_least &&
+          @expected_received_count > 0 &&
+          @actual_received_count >= @expected_received_count
       end
 
       # @private
       def matches_name_but_not_args(message, *args)
-        @message == message and not @argument_expectation.args_match?(*args)
+        @message == message and not @argument_list_matcher.args_match?(*args)
       end
 
       # @private
@@ -248,12 +256,23 @@ MESSAGE
       # @private
       def generate_error
         if similar_messages.empty?
-          @error_generator.raise_expectation_error(@message, @expected_received_count, @actual_received_count, *@argument_expectation.args)
+          @error_generator.raise_expectation_error(@message, @expected_received_count, @argument_list_matcher, @actual_received_count, expectation_count_type, *expected_args)
         else
           @error_generator.raise_similar_message_args_error(self, *@similar_messages)
         end
       end
 
+      def expectation_count_type
+        return :at_least if @at_least
+        return :at_most if @at_most
+        return nil
+      end
+
+      # @private
+      def description
+        @error_generator.describe_expectation(@message, @expected_received_count, @actual_received_count, *expected_args)
+      end
+
       def raise_out_of_order_error
         @error_generator.raise_out_of_order_error @message
       end
@@ -283,8 +302,8 @@ MESSAGE
       #   cart.add(Book.new(:isbn => 1934356379))
       #   # => passes
       def with(*args, &block)
-        @implementation = block if block_given? unless args.empty?
-        @argument_expectation = ArgumentExpectation.new(*args, &block)
+        self.inner_implementation_action = block if block_given? unless args.empty?
+        @argument_list_matcher = ArgumentListMatcher.new(*args, &block)
         self
       end
 
@@ -293,9 +312,9 @@ MESSAGE
       #
       # @example
       #
-      #   dealer.should_recieve(:deal_card).exactly(10).times
+      #   dealer.should_receive(:deal_card).exactly(10).times
       def exactly(n, &block)
-        @implementation = block if block
+        self.inner_implementation_action = block
         set_expected_received_count :exactly, n
         self
       end
@@ -305,9 +324,13 @@ MESSAGE
       #
       # @example
       #
-      #   dealer.should_recieve(:deal_card).at_least(9).times
+      #   dealer.should_receive(:deal_card).at_least(9).times
       def at_least(n, &block)
-        @implementation = block if block
+        if n == 0
+          RSpec.deprecate "at_least(0) with should_receive", :replacement => "stub"
+        end
+
+        self.inner_implementation_action = block
         set_expected_received_count :at_least, n
         self
       end
@@ -317,9 +340,9 @@ MESSAGE
       #
       # @example
       #
-      #   dealer.should_recieve(:deal_card).at_most(10).times
+      #   dealer.should_receive(:deal_card).at_most(10).times
       def at_most(n, &block)
-        @implementation = block if block
+        self.inner_implementation_action = block
         set_expected_received_count :at_most, n
         self
       end
@@ -328,18 +351,19 @@ MESSAGE
       #
       # @example
       #
-      #   dealer.should_recieve(:deal_card).exactly(10).times
-      #   dealer.should_recieve(:deal_card).at_least(10).times
-      #   dealer.should_recieve(:deal_card).at_most(10).times
+      #   dealer.should_receive(:deal_card).exactly(10).times
+      #   dealer.should_receive(:deal_card).at_least(10).times
+      #   dealer.should_receive(:deal_card).at_most(10).times
       def times(&block)
-        @implementation = block if block
+        self.inner_implementation_action = block
         self
       end
 
 
       # Allows an expected message to be received any number of times.
       def any_number_of_times(&block)
-        @implementation = block if block
+        RSpec.deprecate "any_number_of_times", :replacement => "stub"
+        self.inner_implementation_action = block
         @expected_received_count = :any
         self
       end
@@ -350,6 +374,7 @@ MESSAGE
       #
       #   car.should_receive(:stop).never
       def never
+        ErrorGenerator.raise_double_negation_error("expect(obj)") if negative?
         @expected_received_count = 0
         self
       end
@@ -360,7 +385,7 @@ MESSAGE
       #
       #   car.should_receive(:go).once
       def once(&block)
-        @implementation = block if block
+        self.inner_implementation_action = block
         set_expected_received_count :exactly, 1
         self
       end
@@ -371,7 +396,7 @@ MESSAGE
       #
       #   car.should_receive(:go).twice
       def twice(&block)
-        @implementation = block if block
+        self.inner_implementation_action = block
         set_expected_received_count :exactly, 2
         self
       end
@@ -384,7 +409,7 @@ MESSAGE
       #   api.should_receive(:run).ordered
       #   api.should_receive(:finish).ordered
       def ordered(&block)
-        @implementation = block if block
+        self.inner_implementation_action = block
         @order_group.register(self)
         @ordered = true
         self
@@ -392,7 +417,7 @@ MESSAGE
 
       # @private
       def negative_expectation_for?(message)
-        return false
+        @message == message && negative?
       end
 
       # @private
@@ -405,33 +430,7 @@ MESSAGE
         @actual_received_count += 1
       end
 
-      protected
-
-      def call_with_yield(&block)
-        @error_generator.raise_missing_block_error @args_to_yield unless block
-        value = nil
-        @args_to_yield.each do |args|
-          if block.arity > -1 && args.length != block.arity
-            @error_generator.raise_wrong_arity_error args, block.arity
-          end
-          value = @eval_context ? @eval_context.instance_exec(*args, &block) : block.call(*args)
-        end
-        value
-      end
-
-      def call_implementation_consecutive(*args, &block)
-        @value ||= call_implementation(*args, &block)
-        @value[[@actual_received_count, @value.size-1].min]
-      end
-
-      def call_implementation(*args, &block)
-        @implementation.arity == 0 ? @implementation.call(&block) : @implementation.call(*args, &block)
-      end
-
-      def clone_args_to_yield(*args)
-        @args_to_yield = args.clone
-        @args_to_yield_were_cloned = true
-      end
+    private
 
       def failed_fast?
         @failed_fast
@@ -448,32 +447,136 @@ MESSAGE
                                    end
       end
 
-      def clear_actual_received_count!
-        @actual_received_count = 0
+      def initial_implementation_action=(action)
+        implementation.initial_action = action
       end
 
-      private
+      def inner_implementation_action=(action)
+        implementation.inner_action = action if action
+      end
 
-      def build_implementation(values)
-        value = values.size == 1 ? values.first : values
-        lambda { value }
+      def terminal_implementation_action=(action)
+        implementation.terminal_action = action
       end
     end
 
+    # Handles the implementation of an `and_yield` declaration.
     # @private
-    class NegativeMessageExpectation < MessageExpectation
-      # @private
-      def initialize(error_generator, expectation_ordering, expected_from, message, &implementation)
-        super(error_generator, expectation_ordering, expected_from, message, 0, {}, &implementation)
+    class AndYieldImplementation
+      def initialize(args_to_yield, eval_context, error_generator)
+        @args_to_yield = args_to_yield
+        @eval_context = eval_context
+        @error_generator = error_generator
       end
 
-      def and_return(*)
-        # no-op
+      def arity
+        -1
       end
 
-      # @private
-      def negative_expectation_for?(message)
-        return @message == message
+      def call(*args_to_ignore, &block)
+        return if @args_to_yield.empty? && @eval_context.nil?
+
+        @error_generator.raise_missing_block_error @args_to_yield unless block
+        value = nil
+        @args_to_yield.each do |args|
+          if block.arity > -1 && args.length != block.arity
+            @error_generator.raise_wrong_arity_error args, block.arity
+          end
+          value = @eval_context ? @eval_context.instance_exec(*args, &block) : block.call(*args)
+        end
+        value
+      end
+    end
+
+    # Handles the implementation of an `and_return` implementation.
+    # @private
+    class AndReturnImplementation
+      def initialize(values_to_return)
+        @values_to_return = values_to_return
+      end
+
+      def arity
+        -1
+      end
+
+      def call(*args_to_ignore, &block)
+        if @values_to_return.size > 1
+          @values_to_return.shift
+        else
+          @values_to_return.first
+        end
+      end
+    end
+
+    # Represents a configured implementation. Takes into account
+    # any number of sub-implementations.
+    # @private
+    class Implementation
+      attr_accessor :initial_action, :inner_action, :terminal_action
+
+      def call(*args, &block)
+        actions.map do |action|
+          action.call(*arg_slice_for(args, action.arity), &block)
+        end.last
+      end
+
+      def arg_slice_for(args, arity)
+        if arity >= 0
+          args.slice(0, arity)
+        else
+          args
+        end
+      end
+
+      def present?
+        actions.any?
+      end
+
+    private
+
+      def actions
+        [initial_action, inner_action, terminal_action].compact
+      end
+    end
+
+    # Represents an `and_call_original` implementation.
+    # @private
+    class AndCallOriginalImplementation
+      def initialize(method)
+        @method = method
+      end
+
+      CannotModifyFurtherError = Class.new(StandardError)
+
+      def arity
+        @method.arity
+      end
+
+      def initial_action=(value)
+        raise cannot_modify_further_error
+      end
+
+      def inner_action=(value)
+        raise cannot_modify_further_error
+      end
+
+      def terminal_action=(value)
+        raise cannot_modify_further_error
+      end
+
+      def present?
+        true
+      end
+
+      def call(*args, &block)
+        @method.call(*args, &block)
+      end
+
+    private
+
+      def cannot_modify_further_error
+        CannotModifyFurtherError.new "This method has already been configured " +
+          "to call the original implementation, and cannot be modified further."
       end
     end
   end
