@@ -2,31 +2,6 @@ module RSpec
   module Mocks
     # @private
     class Proxy
-      class << self
-        # @private
-        def warn_about_expectations_on_nil
-          defined?(@warn_about_expectations_on_nil) ? @warn_about_expectations_on_nil : true
-        end
-
-        # @private
-        def warn_about_expectations_on_nil=(new_value)
-          @warn_about_expectations_on_nil = new_value
-        end
-
-        # @private
-        def allow_message_expectations_on_nil
-          @warn_about_expectations_on_nil = false
-
-          # ensure nil.rspec_verify is called even if an expectation is not set in the example
-          # otherwise the allowance would effect subsequent examples
-          RSpec::Mocks::space.add(nil) unless RSpec::Mocks::space.nil?
-        end
-
-        # @private
-        def allow_message_expectations_on_nil?
-          !warn_about_expectations_on_nil
-        end
-      end
 
       # @private
       def initialize(object, name=nil, options={})
@@ -64,13 +39,55 @@ module RSpec
       end
 
       # @private
-      def add_message_expectation(location, method_name, opts={}, &block)        
-        method_double[method_name].add_expectation @error_generator, @expectation_ordering, location, opts, &block
+      def add_message_expectation(location, method_name, opts={}, &block)
+        meth_double = method_double[method_name]
+
+        if null_object? && !block
+          meth_double.add_default_stub(@error_generator, @expectation_ordering, location, opts) do
+            @object
+          end
+        end
+
+        meth_double.add_expectation @error_generator, @expectation_ordering, location, opts, &block
       end
 
       # @private
-      def add_negative_message_expectation(location, method_name, &implementation)
-        method_double[method_name].add_negative_expectation @error_generator, @expectation_ordering, location, &implementation
+      def build_expectation(method_name)
+        meth_double = method_double[method_name]
+
+        meth_double.build_expectation(
+          @error_generator,
+          @expectation_ordering
+        )
+      end
+
+      # @private
+      def replay_received_message_on(expectation)
+        expected_method_name = expectation.message
+        meth_double = method_double[expected_method_name]
+
+        if meth_double.expectations.any?
+          @error_generator.raise_expectation_on_mocked_method(expected_method_name)
+        end
+
+        unless null_object? || meth_double.stubs.any?
+          @error_generator.raise_expectation_on_unstubbed_method(expected_method_name)
+        end
+
+        @messages_received.each do |(actual_method_name, args, _)|
+          if expectation.matches?(actual_method_name, *args)
+            expectation.invoke(nil)
+          end
+        end
+      end
+
+      # @private
+      def check_for_unexpected_arguments(expectation)
+        @messages_received.each do |(method_name, args, _)|
+          if expectation.matches_name_but_not_args(method_name, *args)
+            raise_unexpected_message_args_error(expectation, *args)
+          end
+        end
       end
 
       # @private
@@ -93,6 +110,7 @@ module RSpec
       # @private
       def reset
         method_doubles.each {|d| d.reset}
+        @messages_received.clear
       end
 
       # @private
@@ -112,6 +130,7 @@ module RSpec
 
       # @private
       def message_received(message, *args, &block)
+        record_message_received message, *args, &block
         expectation = find_matching_expectation(message, *args)
         stub = find_matching_method_stub(message, *args)
 
@@ -120,15 +139,15 @@ module RSpec
           if expectation = find_almost_matching_expectation(message, *args)
             expectation.advise(*args) unless expectation.expected_messages_received?
           end
-          stub.invoke(*args, &block)
+          stub.invoke(nil, *args, &block)
         elsif expectation
-          expectation.invoke(*args, &block)
+          expectation.invoke(stub, *args, &block)
         elsif expectation = find_almost_matching_expectation(message, *args)
           expectation.advise(*args) if null_object? unless expectation.expected_messages_received?
           raise_unexpected_message_args_error(expectation, *args) unless (has_negative_expectation?(message) or null_object?)
         elsif stub = find_almost_matching_stub(message, *args)
           stub.advise(*args)
-          raise_unexpected_message_args_error(stub, *args)
+          raise_missing_default_stub_error(stub, *args)
         elsif @object.is_a?(Class)
           @object.superclass.__send__(message, *args, &block)
         else
@@ -136,14 +155,19 @@ module RSpec
         end
       end
 
+      # @private
+      def raise_unexpected_message_error(method_name, *args)
+        @error_generator.raise_unexpected_message_error method_name, *args
+      end
+
       # @private
       def raise_unexpected_message_args_error(expectation, *args)
         @error_generator.raise_unexpected_message_args_error(expectation, *args)
       end
 
       # @private
-      def raise_unexpected_message_error(method_name, *args)
-        @error_generator.raise_unexpected_message_error method_name, *args
+      def raise_missing_default_stub_error(expectation, *args)
+        @error_generator.raise_missing_default_stub_error(expectation, *args)
       end
 
       private
@@ -157,12 +181,27 @@ module RSpec
       end
 
       def find_matching_expectation(method_name, *args)
-        method_double[method_name].expectations.find {|expectation| expectation.matches?(method_name, *args) && !expectation.called_max_times?} || 
-          method_double[method_name].expectations.find {|expectation| expectation.matches?(method_name, *args)}
+        find_best_matching_expectation_for(method_name) do |expectation|
+          expectation.matches?(method_name, *args)
+        end
       end
 
       def find_almost_matching_expectation(method_name, *args)
-        method_double[method_name].expectations.find {|expectation| expectation.matches_name_but_not_args(method_name, *args) && !expectation.called_max_times?}
+        find_best_matching_expectation_for(method_name) do |expectation|
+          expectation.matches_name_but_not_args(method_name, *args)
+        end
+      end
+
+      def find_best_matching_expectation_for(method_name)
+        first_match = nil
+
+        method_double[method_name].expectations.each do |expectation|
+          next unless yield expectation
+          return expectation unless expectation.called_max_times?
+          first_match ||= expectation
+        end
+
+        first_match
       end
 
       def find_matching_method_stub(method_name, *args)
