@@ -108,6 +108,13 @@ module RSpec
             klass.any_instance.stub(:foo).and_return(result = Object.new)
             expect(instance.foo).to eq(result)
           end
+
+          it 'handles freeze and duplication correctly' do
+            String.any_instance.stub(:any_method)
+
+            foo = 'foo'.freeze
+            expect(foo.dup.concat 'bar').to eq 'foobar'
+          end
         end
 
         context "with argument matching" do
@@ -174,6 +181,26 @@ module RSpec
           end
         end
 
+        context 'with #and_call_original and competing #with' do
+          let(:klass) { Struct.new(:a_method) }
+
+          it 'can combine and_call_original, with, and_return' do
+            allow_any_instance_of(klass).to receive(:a_method).and_call_original
+            allow_any_instance_of(klass).to receive(:a_method).with(:arg).and_return('value')
+
+            expect(klass.new('org').a_method).to eq 'org'
+            expect(klass.new.a_method(:arg)).to  eq 'value'
+          end
+
+          it 'can combine and_call_original, with, and_return (old syntax)' do
+            klass.any_instance.stub(:a_method).and_call_original
+            klass.any_instance.stub(:a_method).with(:arg).and_return('value')
+
+            expect(klass.new('org').a_method).to eq 'org'
+            expect(klass.new.a_method(:arg)).to  eq 'value'
+          end
+        end
+
         context "with #and_raise" do
           it "stubs a method that doesn't exist" do
             klass.any_instance.stub(:foo).and_raise(CustomErrorForAnyInstanceSpec)
@@ -317,9 +344,15 @@ module RSpec
             klass.any_instance.should_receive(:foo)
             klass.any_instance.should_not_receive(:bar)
             klass.new.foo
-            klass.rspec_verify
+            RSpec::Mocks.space.verify_all
           end
         end
+
+        it "prevents confusing double-negative expressions involving `never`" do
+          expect {
+            klass.any_instance.should_not_receive(:not_expected).never
+          }.to raise_error(/trying to negate it again/)
+        end
       end
 
       context "with #should_receive" do
@@ -336,14 +369,14 @@ module RSpec
             expect do
               klass.any_instance.should_receive(:foo)
               klass.new
-              klass.rspec_verify
+              RSpec::Mocks.space.verify_all
             end.to raise_error(RSpec::Mocks::MockExpectationError, foo_expectation_error_message)
           end
 
           it "fails if no instance is created" do
             expect do
               klass.any_instance.should_receive(:foo).and_return(1)
-              klass.rspec_verify
+              RSpec::Mocks.space.verify_all
             end.to raise_error(RSpec::Mocks::MockExpectationError, foo_expectation_error_message)
           end
 
@@ -351,7 +384,7 @@ module RSpec
             expect do
               klass.any_instance.should_receive(:foo)
               klass.any_instance.should_receive(:bar)
-              klass.rspec_verify
+              RSpec::Mocks.space.verify_all
             end.to raise_error(RSpec::Mocks::MockExpectationError, 'Exactly one instance should have received the following message(s) but didn\'t: bar, foo')
           end
 
@@ -389,7 +422,7 @@ module RSpec
                 klass.any_instance.should_receive(:foo)
                 klass.should_receive(:woot)
                 klass.new.foo
-                klass.rspec_verify
+                RSpec::Mocks.space.verify_all
               end.to(raise_error(RSpec::Mocks::MockExpectationError) do |error|
                 expect(error.message).not_to eq(existing_method_expectation_error_message)
               end)
@@ -415,14 +448,14 @@ module RSpec
             expect do
               klass.any_instance.should_receive(:existing_method)
               klass.new
-              klass.rspec_verify
+              RSpec::Mocks.space.verify_all
             end.to raise_error(RSpec::Mocks::MockExpectationError, existing_method_expectation_error_message)
           end
 
           it "fails if no instance is created" do
             expect do
               klass.any_instance.should_receive(:existing_method)
-              klass.rspec_verify
+              RSpec::Mocks.space.verify_all
             end.to raise_error(RSpec::Mocks::MockExpectationError, existing_method_expectation_error_message)
           end
 
@@ -430,7 +463,7 @@ module RSpec
             expect do
               klass.any_instance.should_receive(:existing_method)
               klass.any_instance.should_receive(:another_existing_method)
-              klass.rspec_verify
+              RSpec::Mocks.space.verify_all
             end.to raise_error(RSpec::Mocks::MockExpectationError, 'Exactly one instance should have received the following message(s) but didn\'t: another_existing_method, existing_method')
           end
 
@@ -493,7 +526,7 @@ module RSpec
             end.to raise_error(RSpec::Mocks::MockExpectationError)
 
             # ignore the fact that should_receive expectations were not met
-            instances.each { |instance| instance.rspec_reset }
+            instances.each { |instance| reset instance }
           end
 
           it "is not affected by the invocation of existing methods on other instances" do
@@ -510,7 +543,7 @@ module RSpec
             end.to raise_error(RSpec::Mocks::MockExpectationError)
 
             # ignore the fact that should_receive expectations were not met
-            instance.rspec_reset
+            reset instance
           end
         end
 
@@ -524,7 +557,7 @@ module RSpec
             it "fails when no instances are declared" do
               expect do
                 klass.any_instance.should_receive(:foo).once
-                klass.rspec_verify
+                RSpec::Mocks.space.verify_all
               end.to raise_error(RSpec::Mocks::MockExpectationError, foo_expectation_error_message)
             end
 
@@ -532,7 +565,7 @@ module RSpec
               expect do
                 klass.any_instance.should_receive(:foo).once
                 klass.new
-                klass.rspec_verify
+                RSpec::Mocks.space.verify_all
               end.to raise_error(RSpec::Mocks::MockExpectationError, foo_expectation_error_message)
             end
 
@@ -541,7 +574,7 @@ module RSpec
                 klass.any_instance.should_receive(:foo).once
                 instance = klass.new
                 2.times { instance.foo }
-                instance.rspec_verify
+                verify instance
               end.to raise_error(RSpec::Mocks::MockExpectationError)
             end
           end
@@ -558,7 +591,7 @@ module RSpec
                 klass.any_instance.should_receive(:foo).twice
                 instance = klass.new
                 3.times { instance.foo }
-                instance.rspec_verify
+                verify instance
               end.to raise_error(RSpec::Mocks::MockExpectationError)
             end
           end
@@ -575,7 +608,7 @@ module RSpec
                 klass.any_instance.should_receive(:foo).exactly(3).times
                 instance = klass.new
                 2.times { instance.foo }
-                instance.rspec_verify
+                verify instance
               end.to raise_error(RSpec::Mocks::MockExpectationError)
             end
 
@@ -584,7 +617,7 @@ module RSpec
                 klass.any_instance.should_receive(:foo).exactly(3).times
                 instance = klass.new
                 4.times { instance.foo }
-                instance.rspec_verify
+                verify instance
               end.to raise_error(RSpec::Mocks::MockExpectationError)
             end
           end
@@ -601,7 +634,7 @@ module RSpec
                 klass.any_instance.should_receive(:foo).at_least(3).times
                 instance = klass.new
                 2.times { instance.foo }
-                instance.rspec_verify
+                verify instance
               end.to raise_error(RSpec::Mocks::MockExpectationError)
             end
 
@@ -630,7 +663,7 @@ module RSpec
                 klass.any_instance.should_receive(:foo).at_most(3).times
                 instance = klass.new
                 4.times { instance.foo }
-                instance.rspec_verify
+                verify instance
               end.to raise_error(RSpec::Mocks::MockExpectationError)
             end
           end
@@ -638,7 +671,7 @@ module RSpec
           context "the 'never' constraint" do
             it "passes for 0 invocations" do
               klass.any_instance.should_receive(:foo).never
-              klass.rspec_verify
+              RSpec::Mocks.space.verify_all
             end
 
             it "fails on the first invocation" do
@@ -659,7 +692,7 @@ module RSpec
                 expect do
                   klass.any_instance.should_receive(:foo).never
                   klass.any_instance.should_receive(:existing_method).and_return(5)
-                  klass.rspec_verify
+                  RSpec::Mocks.space.verify_all
                 end.to raise_error(RSpec::Mocks::MockExpectationError, existing_method_expectation_error_message)
               end
             end
@@ -668,14 +701,16 @@ module RSpec
           context "the 'any_number_of_times' constraint" do
             it "passes for 0 invocations" do
               klass.any_instance.should_receive(:foo).any_number_of_times
-              klass.new.rspec_verify
+              verify klass.new
             end
 
             it "passes for a non-zero number of invocations" do
+              allow(RSpec).to receive(:deprecate).with("any_number_of_times", :replacement => "stub")
+
               klass.any_instance.should_receive(:foo).any_number_of_times
               instance = klass.new
               instance.foo
-              instance.rspec_verify
+              verify instance
             end
 
             it "does not interfere with other expectations" do
@@ -695,7 +730,7 @@ module RSpec
                 expect do
                   klass.any_instance.should_receive(:foo).any_number_of_times
                   klass.any_instance.should_receive(:existing_method).and_return(5)
-                  klass.rspec_verify
+                  RSpec::Mocks.space.verify_all
                 end.to raise_error(RSpec::Mocks::MockExpectationError, existing_method_expectation_error_message)
               end
             end
@@ -704,11 +739,11 @@ module RSpec
       end
 
       context "when resetting post-verification" do
-        let(:space) { RSpec::Mocks::Space.new }
+        let(:space) { RSpec::Mocks.space }
 
         context "existing method" do
           before(:each) do
-            space.add(klass)
+            klass.any_instance # to force it to be tracked
           end
 
           context "with stubbing" do
@@ -829,31 +864,34 @@ module RSpec
             klass.any_instance.stub(:existing_method).and_return(false)
             klass.any_instance.stub(:existing_method).and_return(true)
 
-            klass.rspec_verify
+            RSpec::Mocks.space.verify_all
             expect(klass.new).to respond_to(:existing_method)
             expect(klass.new.existing_method).to eq(existing_method_return_value)
           end
         end
 
         it "adds an class to the current space when #any_instance is invoked" do
-          klass.any_instance
-          expect(RSpec::Mocks::space.send(:receivers)).to include(klass)
+          expect {
+            klass.any_instance
+          }.to change { space.any_instance_recorders.size }.by(1)
         end
 
         it "adds an instance to the current space when stubbed method is invoked" do
           klass.any_instance.stub(:foo)
           instance = klass.new
           instance.foo
-          expect(RSpec::Mocks::space.send(:receivers)).to include(instance)
+          expect(RSpec::Mocks.space.proxies.keys).to include(instance.object_id)
         end
       end
 
       context 'when used in conjunction with a `dup`' do
         it "doesn't cause an infinite loop" do
+          pending "This intermittently fails on JRuby" if RUBY_PLATFORM == 'java'
+
           Object.any_instance.stub(:some_method)
           o = Object.new
           o.some_method
-          expect { o.dup.some_method }.to_not raise_error(SystemStackError)
+          expect { o.dup.some_method }.to_not raise_error
         end
 
         it "doesn't bomb if the object doesn't support `dup`" do
@@ -871,7 +909,7 @@ module RSpec
 
           klass.any_instance
 
-          expect { klass.new.dup('Dup dup dup') }.to_not raise_error(ArgumentError)
+          expect { klass.new.dup('Dup dup dup') }.to_not raise_error
         end
       end
 
