@@ -5,7 +5,7 @@ module RSpec
 
     describe "ordering" do
       before { @double = double("test double") }
-      after  { @double.rspec_reset }
+      after  { reset @double }
 
       it "passes when messages are received in order" do
         @double.should_receive(:one).ordered
@@ -30,9 +30,9 @@ module RSpec
       it "fails when messages are received out of order (2nd message 1st)" do
         @double.should_receive(:one).ordered
         @double.should_receive(:two).ordered
-        lambda do
+        expect {
           @double.two
-        end.should raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :two out of order")
+        }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :two out of order")
       end
 
       it "fails when messages are received out of order (3rd message 1st)" do
@@ -40,9 +40,9 @@ module RSpec
         @double.should_receive(:two).ordered
         @double.should_receive(:three).ordered
         @double.one
-        lambda do
+        expect {
           @double.three
-        end.should raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :three out of order")
+        }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :three out of order")
       end
 
       it "fails when messages are received out of order (3rd message 2nd)" do
@@ -50,9 +50,9 @@ module RSpec
         @double.should_receive(:two).ordered
         @double.should_receive(:three).ordered
         @double.one
-        lambda do
+        expect {
           @double.three
-        end.should raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :three out of order")
+        }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :three out of order")
       end
 
       it "fails when messages are out of order across objects" do
@@ -62,11 +62,11 @@ module RSpec
         b.should_receive(:two).ordered
         a.should_receive(:three).ordered
         a.one
-        lambda do
+        expect {
           a.three
-        end.should raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :three out of order")
-        a.rspec_reset
-        b.rspec_reset
+        }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :three out of order")
+        reset a
+        reset b
       end
 
       it "ignores order of non ordered messages" do
@@ -86,7 +86,7 @@ module RSpec
         @double.ignored_2
         @double.ordered_3
         @double.ignored_1
-        @double.rspec_verify
+        verify @double
       end
 
       it "supports duplicate messages" do
