@@ -3,19 +3,25 @@ require 'spec_helper'
 module RSpec
   module Mocks
     describe Mock do
-      treats_method_missing_as_private :subject => RSpec::Mocks::Mock.new, :noop => false
-
       before(:each) { @double = double("test double") }
-      after(:each)  { @double.rspec_reset }
+      after(:each)  { reset @double }
+
+      it "has method_missing as private" do
+        expect(RSpec::Mocks::Mock.private_instance_methods).to include_method(:method_missing)
+      end
+
+      it "does not respond_to? method_missing (because it's private)" do
+        expect(RSpec::Mocks::Mock.new).not_to respond_to(:method_missing)
+      end
 
       it "reports line number of expectation of unreceived message" do
         expected_error_line = __LINE__; @double.should_receive(:wont_happen).with("x", 3)
         begin
-          @double.rspec_verify
+          verify @double
           violated
         rescue RSpec::Mocks::MockExpectationError => e
           # NOTE - this regexp ended w/ $, but jruby adds extra info at the end of the line
-          e.backtrace[0].should match(/#{File.basename(__FILE__)}:#{expected_error_line}/)
+          expect(e.backtrace[0]).to match(/#{File.basename(__FILE__)}:#{expected_error_line}/)
         end
       end
 
@@ -23,31 +29,54 @@ module RSpec
         @double.stub(:wont_happen)
         expected_error_line = __LINE__; @double.should_receive(:wont_happen).with("x", 3)
         begin
-          @double.rspec_verify
+          verify @double
           violated
         rescue RSpec::Mocks::MockExpectationError => e
           # NOTE - this regexp ended w/ $, but jruby adds extra info at the end of the line
-          e.backtrace[0].should match(/#{File.basename(__FILE__)}:#{expected_error_line}/)
+          expect(e.backtrace[0]).to match(/#{File.basename(__FILE__)}:#{expected_error_line}/)
         end
       end
 
       it "passes when not receiving message specified as not to be received" do
         @double.should_not_receive(:not_expected)
-        @double.rspec_verify
+        verify @double
       end
 
-      it "passes when not receiving message specified as not to be received with and_return" do
-        # NOTE (DC 2012-05-05) calling `and_return` after `should_not_receive` makes no sense
-        # and should probably be disallowed.
-        @double.should_not_receive(:not_expected).and_return nil
-        @double.rspec_verify
+      it "prevents confusing double-negative expressions involving `never`" do
+        expect {
+          @double.should_not_receive(:not_expected).never
+        }.to raise_error(/trying to negate it again/)
+      end
+
+      def expect_and_return_warning
+        expect(RSpec).to receive(:deprecate).with(/`and_return` on a negative message expectation/)
+      end
+
+      it "warns when `should_not_receive().and_return` is used" do
+        expect_and_return_warning
+        @double.should_not_receive(:foo).and_return(1)
+      end
+
+      it "warns when `should_receive().never.and_return` is used" do
+        expect_and_return_warning
+        @double.should_receive(:foo).never.and_return(1)
+      end
+
+      it "warns when `expect().not_to receive().and_return` is used" do
+        expect_and_return_warning
+        expect(@double).not_to receive(:foo).and_return(1)
+      end
+
+      it "warns when `expect().to receive().never.and_return` is used" do
+        expect_and_return_warning
+        expect(@double).to receive(:foo).never.and_return(1)
       end
 
       it "passes when receiving message specified as not to be received with different args" do
         @double.should_not_receive(:message).with("unwanted text")
         @double.should_receive(:message).with("other text")
         @double.message "other text"
-        @double.rspec_verify
+        verify @double
       end
 
       it "fails when receiving message specified as not to be received" do
@@ -57,7 +86,7 @@ module RSpec
           violated
         }.to raise_error(
           RSpec::Mocks::MockExpectationError,
-          %Q|(Double "test double").not_expected(no args)\n    expected: 0 times\n    received: 1 time|
+          %Q|(Double "test double").not_expected(no args)\n    expected: 0 times with any arguments\n    received: 1 time|
         )
       end
 
@@ -68,76 +97,91 @@ module RSpec
           violated
         }.to raise_error(
           RSpec::Mocks::MockExpectationError,
-          %Q|(Double "test double").not_expected("unexpected text")\n    expected: 0 times\n    received: 1 time|
+          %Q|(Double "test double").not_expected("unexpected text")\n    expected: 0 times with arguments: ("unexpected text")\n    received: 1 time with arguments: ("unexpected text")|
+        )
+      end
+
+      it "fails when array arguments do not match" do
+        @double.should_not_receive(:not_expected).with(["do not want"])
+        expect {
+          @double.not_expected(["do not want"])
+          violated
+        }.to raise_error(
+          RSpec::Mocks::MockExpectationError,
+          %Q|(Double "test double").not_expected(["do not want"])\n    expected: 0 times with arguments: (["do not want"])\n    received: 1 time with arguments: (["do not want"])|
         )
       end
 
       it "passes when receiving message specified as not to be received with wrong args" do
         @double.should_not_receive(:not_expected).with("unexpected text")
         @double.not_expected "really unexpected text"
-        @double.rspec_verify
+        verify @double
+
+        @double.should_receive(:not_expected).with("unexpected text").never
+        @double.not_expected "really unexpected text"
+        verify @double
       end
 
       it "allows block to calculate return values" do
         @double.should_receive(:something).with("a","b","c").and_return { |a,b,c| c+b+a }
-        @double.something("a","b","c").should eq "cba"
-        @double.rspec_verify
+        expect(@double.something("a","b","c")).to eq "cba"
+        verify @double
       end
 
       it "allows parameter as return value" do
         @double.should_receive(:something).with("a","b","c").and_return("booh")
-        @double.something("a","b","c").should eq "booh"
-        @double.rspec_verify
+        expect(@double.something("a","b","c")).to eq "booh"
+        verify @double
       end
 
       it "returns the previously stubbed value if no return value is set" do
         @double.stub(:something).with("a","b","c").and_return(:stubbed_value)
         @double.should_receive(:something).with("a","b","c")
-        @double.something("a","b","c").should eq :stubbed_value
-        @double.rspec_verify
+        expect(@double.something("a","b","c")).to eq :stubbed_value
+        verify @double
       end
 
       it "returns nil if no return value is set and there is no previously stubbed value" do
         @double.should_receive(:something).with("a","b","c")
-        @double.something("a","b","c").should be_nil
-        @double.rspec_verify
+        expect(@double.something("a","b","c")).to be_nil
+        verify @double
       end
 
       it "raises exception if args don't match when method called" do
         @double.should_receive(:something).with("a","b","c").and_return("booh")
-        lambda {
+        expect {
           @double.something("a","d","c")
           violated
-        }.should raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :something with unexpected arguments\n  expected: (\"a\", \"b\", \"c\")\n       got: (\"a\", \"d\", \"c\")")
+        }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :something with unexpected arguments\n  expected: (\"a\", \"b\", \"c\")\n       got: (\"a\", \"d\", \"c\")")
       end
 
       describe "even when a similar expectation with different arguments exist" do
         it "raises exception if args don't match when method called, correctly reporting the offending arguments" do
           @double.should_receive(:something).with("a","b","c").once
           @double.should_receive(:something).with("z","x","c").once
-          lambda {
+          expect {
             @double.something("a","b","c")
             @double.something("z","x","g")
-          }.should raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :something with unexpected arguments\n  expected: (\"z\", \"x\", \"c\")\n       got: (\"z\", \"x\", \"g\")")
+          }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :something with unexpected arguments\n  expected: (\"z\", \"x\", \"c\")\n       got: (\"z\", \"x\", \"g\")")
         end
       end
 
       it "raises exception if args don't match when method called even when the method is stubbed" do
         @double.stub(:something)
         @double.should_receive(:something).with("a","b","c")
-        lambda {
+        expect {
           @double.something("a","d","c")
-          @double.rspec_verify
-        }.should raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :something with unexpected arguments\n  expected: (\"a\", \"b\", \"c\")\n       got: (\"a\", \"d\", \"c\")")
+          verify @double
+        }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :something with unexpected arguments\n  expected: (\"a\", \"b\", \"c\")\n       got: (\"a\", \"d\", \"c\")")
       end
 
       it "raises exception if args don't match when method called even when using null_object" do
         @double = double("test double").as_null_object
         @double.should_receive(:something).with("a","b","c")
-        lambda {
+        expect {
           @double.something("a","d","c")
-          @double.rspec_verify
-        }.should raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :something with unexpected arguments\n  expected: (\"a\", \"b\", \"c\")\n       got: (\"a\", \"d\", \"c\")")
+          verify @double
+        }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :something with unexpected arguments\n  expected: (\"a\", \"b\", \"c\")\n       got: (\"a\", \"d\", \"c\")")
       end
 
       describe 'with a method that has a default argument' do
@@ -147,117 +191,152 @@ module RSpec
 
           expect {
             @double.method_with_default_argument(nil)
-            @double.rspec_verify
+            verify @double
           }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :method_with_default_argument with unexpected arguments\n  expected: ({})\n       got: (nil)")
         end
       end
 
       it "fails if unexpected method called" do
-        lambda {
+        expect {
           @double.something("a","b","c")
           violated
-        }.should raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received unexpected message :something with (\"a\", \"b\", \"c\")")
+        }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received unexpected message :something with (\"a\", \"b\", \"c\")")
       end
 
       it "uses block for expectation if provided" do
         @double.should_receive(:something) do | a, b |
-          a.should eq "a"
-          b.should eq "b"
+          expect(a).to eq "a"
+          expect(b).to eq "b"
           "booh"
         end
-        @double.something("a", "b").should eq "booh"
-        @double.rspec_verify
+        expect(@double.something("a", "b")).to eq "booh"
+        verify @double
       end
 
       it "fails if expectation block fails" do
-        @double.should_receive(:something) {| bool | bool.should be_true}
+        @double.should_receive(:something) do |bool|
+          expect(bool).to be_true
+        end
+
         expect {
           @double.something false
         }.to raise_error(RSpec::Expectations::ExpectationNotMetError)
       end
 
-      it "passes proc to expectation block without an argument", :ruby => '> 1.8.6' do
-        # We eval this because Ruby 1.8.6's syntax parser barfs on { |&block| ... }
-        # and prevents the entire spec suite from running.
-        eval("@double.should_receive(:foo) {|&block| block.call.should eq(:bar)}")
+      it "is wrappable in an array" do
+        expect( Array(@double) ).to eq([@double])
+      end
+
+      it "is wrappable in an array when a null object" do
+        expect( Array(@double.as_null_object) ).to eq [@double]
+      end
+
+      it "responds to to_ary as a null object" do
+        expect(@double.as_null_object.to_ary).to eq nil
+      end
+
+      it "responds to to_a as a null object" do
+        if RUBY_VERSION.to_f > 1.8
+          expect(@double.as_null_object.to_a).to eq nil
+        else
+          expect(@double.as_null_object.to_a).to eq [@double]
+        end
+      end
+
+      it "passes proc to expectation block without an argument" do
+        @double.should_receive(:foo) { |&block| expect(block.call).to eq(:bar) }
         @double.foo { :bar }
       end
 
-      it "passes proc to expectation block with an argument", :ruby => '> 1.8.6' do
-        eval("@double.should_receive(:foo) {|arg, &block| block.call.should eq(:bar)}")
-        @double.foo(:arg) { :bar }
-      end
+      context "with Ruby > 1.8.6", :unless => RUBY_VERSION.to_s == '1.8.6' do
+        it "passes proc to expectation block without an argument" do
+          # We eval this because Ruby 1.8.6's syntax parser barfs on { |&block| ... }
+          # and prevents the entire spec suite from running.
+          eval("@double.should_receive(:foo) {|&block| expect(block.call).to eq(:bar)}")
+          @double.foo { :bar }
+        end
 
-      it "passes proc to stub block without an argurment", :ruby => '>1.8.6' do
-        eval("@double.stub(:foo) {|&block| block.call.should eq(:bar)}")
-        @double.foo { :bar }
-      end
+        it "passes proc to expectation block with an argument" do
+          eval("@double.should_receive(:foo) {|arg, &block| expect(block.call).to eq(:bar)}")
+          @double.foo(:arg) { :bar }
+        end
 
-      it "passes proc to stub block with an argument", :ruby => '> 1.8.6' do
-        eval("@double.stub(:foo) {|arg, &block| block.call.should eq(:bar)}")
-        @double.foo(:arg) { :bar }
+        it "passes proc to stub block without an argurment" do
+          eval("@double.stub(:foo) {|&block| expect(block.call).to eq(:bar)}")
+          @double.foo { :bar }
+        end
+
+        it "passes proc to stub block with an argument" do
+          eval("@double.stub(:foo) {|arg, &block| expect(block.call).to eq(:bar)}")
+          @double.foo(:arg) { :bar }
+        end
       end
 
       it "fails right away when method defined as never is received" do
         @double.should_receive(:not_expected).never
         expect { @double.not_expected }.
           to raise_error(RSpec::Mocks::MockExpectationError,
-                         %Q|(Double "test double").not_expected(no args)\n    expected: 0 times\n    received: 1 time|
+                         %Q|(Double "test double").not_expected(no args)\n    expected: 0 times with any arguments\n    received: 1 time|
         )
       end
 
-      it "raises when told to" do
-        @double.should_receive(:something).and_raise(RuntimeError)
+      it "raises RuntimeError by default" do
+        @double.should_receive(:something).and_raise
         expect { @double.something }.to raise_error(RuntimeError)
       end
 
-      it "raises instance of submitted Exception" do
-        error = RuntimeError.new("error message")
-        @double.should_receive(:something).and_raise(error)
-        lambda {
-          @double.something
-        }.should raise_error(RuntimeError, "error message")
-      end
-
-      it "raises instance of submitted ArgumentError" do
-        error = ArgumentError.new("error message")
-        @double.should_receive(:something).and_raise(error)
-        lambda {
-          @double.something
-        }.should raise_error(ArgumentError, "error message")
-      end
-
-      it "fails with helpful message if submitted Exception requires constructor arguments" do
-        class ErrorWithNonZeroArgConstructor < RuntimeError
-          def initialize(i_take_an_argument)
-          end
-        end
-
-        @double.stub(:something).and_raise(ErrorWithNonZeroArgConstructor)
-        lambda {
-          @double.something
-        }.should raise_error(ArgumentError, /^'and_raise' can only accept an Exception class if an instance/)
-      end
-
-      it "raises RuntimeError with submitted message" do
+      it "raises RuntimeError with a message by default" do
         @double.should_receive(:something).and_raise("error message")
-        lambda {
+        expect { @double.something }.to raise_error(RuntimeError, "error message")
+      end
+
+      it "raises an exception of a given type without an error message" do
+        @double.should_receive(:something).and_raise(StandardError)
+        expect { @double.something }.to raise_error(StandardError)
+      end
+
+      it "raises an exception of a given type with a message" do
+        @double.should_receive(:something).and_raise(RuntimeError, "error message")
+        expect { @double.something }.to raise_error(RuntimeError, "error message")
+      end
+
+      it "raises a given instance of an exception" do
+        @double.should_receive(:something).and_raise(RuntimeError.new("error message"))
+        expect { @double.something }.to raise_error(RuntimeError, "error message")
+      end
+
+      class OutOfGas < StandardError
+        attr_reader :amount, :units
+        def initialize(amount, units)
+          @amount = amount
+          @units  = units
+        end
+      end
+
+      it "raises a given instance of an exception with arguments other than the standard 'message'" do
+        @double.should_receive(:something).and_raise(OutOfGas.new(2, :oz))
+
+        begin
           @double.something
-        }.should raise_error(RuntimeError, "error message")
+          fail "OutOfGas was not raised"
+        rescue OutOfGas => e
+          expect(e.amount).to eq 2
+          expect(e.units).to eq :oz
+        end
       end
 
       it "does not raise when told to if args dont match" do
         @double.should_receive(:something).with(2).and_raise(RuntimeError)
-        lambda {
+        expect {
           @double.something 1
-        }.should raise_error(RSpec::Mocks::MockExpectationError)
+        }.to raise_error(RSpec::Mocks::MockExpectationError)
       end
 
       it "throws when told to" do
         @double.should_receive(:something).and_throw(:blech)
-        lambda {
+        expect {
           @double.something
-        }.should throw_symbol(:blech)
+        }.to throw_symbol(:blech)
       end
 
       it "ignores args on any args" do
@@ -266,35 +345,36 @@ module RSpec
         @double.something 1
         @double.something "a", 2
         @double.something [], {}, "joe", 7
-        @double.rspec_verify
+        verify @double
       end
 
       it "fails on no args if any args received" do
         @double.should_receive(:something).with(no_args())
-        lambda {
+        expect {
           @double.something 1
-        }.should raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :something with unexpected arguments\n  expected: (no args)\n       got: (1)")
+        }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :something with unexpected arguments\n  expected: (no args)\n       got: (1)")
       end
 
       it "fails when args are expected but none are received" do
         @double.should_receive(:something).with(1)
-        lambda {
+        expect {
           @double.something
-        }.should raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :something with unexpected arguments\n  expected: (1)\n       got: (no args)")
+        }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" received :something with unexpected arguments\n  expected: (1)\n       got: (no args)")
       end
 
       it "returns value from block by default" do
         @double.stub(:method_that_yields).and_yield
-        @double.method_that_yields { :returned_obj }.should eq :returned_obj
-        @double.rspec_verify
+        value = @double.method_that_yields { :returned_obj }
+        expect(value).to eq :returned_obj
+        verify @double
       end
 
       it "yields 0 args to blocks that take a variable number of arguments" do
         @double.should_receive(:yield_back).with(no_args()).once.and_yield
         a = nil
         @double.yield_back {|*x| a = x}
-        a.should eq []
-        @double.rspec_verify
+        expect(a).to eq []
+        verify @double
       end
 
       it "yields 0 args multiple times to blocks that take a variable number of arguments" do
@@ -302,16 +382,16 @@ module RSpec
                                                                     and_yield
         b = []
         @double.yield_back {|*a| b << a}
-        b.should eq [ [], [] ]
-        @double.rspec_verify
+        expect(b).to eq [ [], [] ]
+        verify @double
       end
 
       it "yields one arg to blocks that take a variable number of arguments" do
         @double.should_receive(:yield_back).with(no_args()).once.and_yield(99)
         a = nil
         @double.yield_back {|*x| a = x}
-        a.should eq [99]
-        @double.rspec_verify
+        expect(a).to eq [99]
+        verify @double
       end
 
       it "yields one arg 3 times consecutively to blocks that take a variable number of arguments" do
@@ -320,16 +400,16 @@ module RSpec
                                                                     and_yield("something fruity")
         b = []
         @double.yield_back {|*a| b << a}
-        b.should eq [[99], [43], ["something fruity"]]
-        @double.rspec_verify
+        expect(b).to eq [[99], [43], ["something fruity"]]
+        verify @double
       end
 
       it "yields many args to blocks that take a variable number of arguments" do
         @double.should_receive(:yield_back).with(no_args()).once.and_yield(99, 27, "go")
         a = nil
         @double.yield_back {|*x| a = x}
-        a.should eq [99, 27, "go"]
-        @double.rspec_verify
+        expect(a).to eq [99, 27, "go"]
+        verify @double
       end
 
       it "yields many args 3 times consecutively to blocks that take a variable number of arguments" do
@@ -338,16 +418,16 @@ module RSpec
                                                                     and_yield("stop", 12, :red)
         b = []
         @double.yield_back {|*a| b << a}
-        b.should eq [[99, :green, "go"], ["wait", :amber], ["stop", 12, :red]]
-        @double.rspec_verify
+        expect(b).to eq [[99, :green, "go"], ["wait", :amber], ["stop", 12, :red]]
+        verify @double
       end
 
       it "yields single value" do
         @double.should_receive(:yield_back).with(no_args()).once.and_yield(99)
         a = nil
         @double.yield_back {|x| a = x}
-        a.should eq 99
-        @double.rspec_verify
+        expect(a).to eq 99
+        verify @double
       end
 
       it "yields single value 3 times consecutively" do
@@ -356,17 +436,17 @@ module RSpec
                                                                     and_yield("something fruity")
         b = []
         @double.yield_back {|a| b << a}
-        b.should eq [99, 43, "something fruity"]
-        @double.rspec_verify
+        expect(b).to eq [99, 43, "something fruity"]
+        verify @double
       end
 
       it "yields two values" do
         @double.should_receive(:yield_back).with(no_args()).once.and_yield('wha', 'zup')
         a, b = nil
         @double.yield_back {|x,y| a=x; b=y}
-        a.should eq 'wha'
-        b.should eq 'zup'
-        @double.rspec_verify
+        expect(a).to eq 'wha'
+        expect(b).to eq 'zup'
+        verify @double
       end
 
       it "yields two values 3 times consecutively" do
@@ -375,66 +455,66 @@ module RSpec
                                                                     and_yield(14, 65)
         c = []
         @double.yield_back {|a,b| c << [a, b]}
-        c.should eq [['wha', 'zup'], ['not', 'down'], [14, 65]]
-        @double.rspec_verify
+        expect(c).to eq [['wha', 'zup'], ['not', 'down'], [14, 65]]
+        verify @double
       end
 
       it "fails when calling yielding method with wrong arity" do
         @double.should_receive(:yield_back).with(no_args()).once.and_yield('wha', 'zup')
-        lambda {
+        expect {
           @double.yield_back {|a|}
-        }.should raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" yielded |\"wha\", \"zup\"| to block with arity of 1")
+        }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" yielded |\"wha\", \"zup\"| to block with arity of 1")
       end
 
       it "fails when calling yielding method consecutively with wrong arity" do
         @double.should_receive(:yield_back).once.with(no_args()).once.and_yield('wha', 'zup').
                                                                     and_yield('down').
                                                                     and_yield(14, 65)
-        lambda {
+        expect {
           c = []
           @double.yield_back {|a,b| c << [a, b]}
-        }.should raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" yielded |\"down\"| to block with arity of 2")
+        }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" yielded |\"down\"| to block with arity of 2")
       end
 
       it "fails when calling yielding method without block" do
         @double.should_receive(:yield_back).with(no_args()).once.and_yield('wha', 'zup')
-        lambda {
+        expect {
           @double.yield_back
-        }.should raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" asked to yield |[\"wha\", \"zup\"]| but no block was passed")
+        }.to raise_error(RSpec::Mocks::MockExpectationError, "Double \"test double\" asked to yield |[\"wha\", \"zup\"]| but no block was passed")
       end
 
       it "is able to double send" do
         @double.should_receive(:send).with(any_args)
         @double.send 'hi'
-        @double.rspec_verify
+        verify @double
       end
 
       it "is able to raise from method calling yielding double" do
         @double.should_receive(:yield_me).and_yield 44
 
-        lambda {
+        expect {
           @double.yield_me do |x|
             raise "Bang"
           end
-        }.should raise_error(StandardError, "Bang")
+        }.to raise_error(StandardError, "Bang")
 
-        @double.rspec_verify
+        verify @double
       end
 
       it "clears expectations after verify" do
         @double.should_receive(:foobar)
         @double.foobar
-        @double.rspec_verify
-        lambda {
+        verify @double
+        expect {
           @double.foobar
-        }.should raise_error(RSpec::Mocks::MockExpectationError, %q|Double "test double" received unexpected message :foobar with (no args)|)
+        }.to raise_error(RSpec::Mocks::MockExpectationError, %q|Double "test double" received unexpected message :foobar with (no args)|)
       end
 
       it "restores objects to their original state on rspec_reset" do
         double = double("this is a double")
         double.should_receive(:blah)
-        double.rspec_reset
-        double.rspec_verify #should throw if reset didn't work
+        reset double
+        verify double #should throw if reset didn't work
       end
 
       it "works even after method_missing starts raising NameErrors instead of NoMethodErrors" do
@@ -460,28 +540,27 @@ module RSpec
 
         @double.should_receive(:foobar)
         @double.foobar
-        @double.rspec_verify
+        verify @double
 
-        lambda { @double.foobar }.should_not raise_error(NameError)
-        lambda { @double.foobar }.should raise_error(RSpec::Mocks::MockExpectationError)
+        expect { @double.foobar }.to raise_error(RSpec::Mocks::MockExpectationError)
       end
 
       it "temporarily replaces a method stub on a double" do
         @double.stub(:msg).and_return(:stub_value)
         @double.should_receive(:msg).with(:arg).and_return(:double_value)
-        @double.msg(:arg).should equal(:double_value)
-        @double.msg.should equal(:stub_value)
-        @double.msg.should equal(:stub_value)
-        @double.rspec_verify
+        expect(@double.msg(:arg)).to equal(:double_value)
+        expect(@double.msg).to equal(:stub_value)
+        expect(@double.msg).to equal(:stub_value)
+        verify @double
       end
 
       it "does not require a different signature to replace a method stub" do
         @double.stub(:msg).and_return(:stub_value)
         @double.should_receive(:msg).and_return(:double_value)
-        @double.msg(:arg).should equal(:double_value)
-        @double.msg.should equal(:stub_value)
-        @double.msg.should equal(:stub_value)
-        @double.rspec_verify
+        expect(@double.msg(:arg)).to equal(:double_value)
+        expect(@double.msg).to equal(:stub_value)
+        expect(@double.msg).to equal(:stub_value)
+        verify @double
       end
 
       it "raises an error when a previously stubbed method has a negative expectation" do
@@ -494,47 +573,47 @@ module RSpec
         non_double = Object.new
         non_double.stub(:msg).and_return(:stub_value)
         non_double.should_receive(:msg).with(:arg).and_return(:double_value)
-        non_double.msg(:arg).should equal(:double_value)
-        non_double.msg.should equal(:stub_value)
-        non_double.msg.should equal(:stub_value)
-        non_double.rspec_verify
+        expect(non_double.msg(:arg)).to equal(:double_value)
+        expect(non_double.msg).to equal(:stub_value)
+        expect(non_double.msg).to equal(:stub_value)
+        verify non_double
       end
 
       it "returns the stubbed value when no new value specified" do
         @double.stub(:msg).and_return(:stub_value)
         @double.should_receive(:msg)
-        @double.msg.should equal(:stub_value)
-        @double.rspec_verify
+        expect(@double.msg).to equal(:stub_value)
+        verify @double
       end
 
       it "returns the stubbed value when stubbed with args and no new value specified" do
         @double.stub(:msg).with(:arg).and_return(:stub_value)
         @double.should_receive(:msg).with(:arg)
-        @double.msg(:arg).should equal(:stub_value)
-        @double.rspec_verify
+        expect(@double.msg(:arg)).to equal(:stub_value)
+        verify @double
       end
 
       it "does not mess with the stub's yielded values when also doubleed" do
         @double.stub(:yield_back).and_yield(:stub_value)
         @double.should_receive(:yield_back).and_yield(:double_value)
-        @double.yield_back{|v| v.should eq :double_value }
-        @double.yield_back{|v| v.should eq :stub_value }
-        @double.rspec_verify
+        @double.yield_back{|v| expect(v).to eq :double_value }
+        @double.yield_back{|v| expect(v).to eq :stub_value }
+        verify @double
       end
 
-      it "yields multiple values after a similar stub" do
-        File.stub(:open).and_yield(:stub_value)
-        File.should_receive(:open).and_yield(:first_call).and_yield(:second_call)
-        yielded_args = []
-        File.open {|v| yielded_args << v }
-        yielded_args.should eq [:first_call, :second_call]
-        File.open {|v| v.should eq :stub_value }
-        File.rspec_verify
+      it "can yield multiple times when told to do so" do
+        @double.stub(:foo).and_yield(:stub_value)
+        @double.should_receive(:foo).and_yield(:first_yield).and_yield(:second_yield)
+
+        expect { |b| @double.foo(&b) }.to yield_successive_args(:first_yield, :second_yield)
+        expect { |b| @double.foo(&b) }.to yield_with_args(:stub_value)
+
+        verify @double
       end
 
       it "assigns stub return values" do
         double = RSpec::Mocks::Mock.new('name', :message => :response)
-        double.message.should eq :response
+        expect(double.message).to eq :response
       end
 
     end
@@ -554,7 +633,7 @@ module RSpec
 
         @double.foo
 
-        @calls.should eq 1
+        expect(@calls).to eq 1
       end
 
       it "calls the block after #should_receive after a similar stub" do
@@ -563,7 +642,7 @@ module RSpec
 
         @double.foo
 
-        @calls.should eq 1
+        expect(@calls).to eq 1
       end
 
       it "calls the block after #once" do
@@ -571,7 +650,7 @@ module RSpec
 
         @double.foo
 
-        @calls.should eq 1
+        expect(@calls).to eq 1
       end
 
       it "calls the block after #twice" do
@@ -580,7 +659,7 @@ module RSpec
         @double.foo
         @double.foo
 
-        @calls.should eq 2
+        expect(@calls).to eq 2
       end
 
       it "calls the block after #times" do
@@ -588,15 +667,17 @@ module RSpec
 
         (1..10).each { @double.foo }
 
-        @calls.should eq 10
+        expect(@calls).to eq 10
       end
 
       it "calls the block after #any_number_of_times" do
+        expect(RSpec).to receive(:deprecate).with("any_number_of_times", :replacement => "stub")
+
         @double.should_receive(:foo).any_number_of_times { add_call }
 
         (1..7).each { @double.foo }
 
-        @calls.should eq 7
+        expect(@calls).to eq 7
       end
 
       it "calls the block after #ordered" do
@@ -606,7 +687,7 @@ module RSpec
         @double.foo
         @double.bar
 
-        @calls.should eq 2
+        expect(@calls).to eq 2
       end
     end
 
@@ -614,14 +695,14 @@ module RSpec
       it 'does not contain < because that might lead to invalid HTML in some situations' do
         double = double("Dog")
         valid_html_str = "#{double}"
-        valid_html_str.should_not include('<')
+        expect(valid_html_str).not_to include('<')
       end
     end
 
     describe "string representation generated by #to_str" do
       it "looks the same as #to_s" do
         double = double("Foo")
-        double.to_str.should eq double.to_s
+        expect(double.to_str).to eq double.to_s
       end
     end
 
@@ -633,8 +714,8 @@ module RSpec
 
       it "does respond to initially stubbed methods" do
         double = double(:foo => "woo", :bar => "car")
-        double.foo.should eq "woo"
-        double.bar.should eq "car"
+        expect(double.foo).to eq "woo"
+        expect(double.bar).to eq "car"
       end
     end
 
@@ -664,7 +745,7 @@ module RSpec
             expect do
               @double.foo('baz')
             end.to raise_error
-            @double.rspec_reset
+            reset @double
           end
         end
 
@@ -676,7 +757,7 @@ module RSpec
             expect do
               @double.foo(d2)
             end.to raise_error
-            @double.rspec_reset
+            reset @double
           end
         end
 
@@ -688,7 +769,7 @@ module RSpec
             expect do
               @double.foo(d2)
             end.to raise_error
-            @double.rspec_reset
+            reset @double
           end
         end
       end
@@ -697,7 +778,7 @@ module RSpec
         context "with matching args" do
           it "returns the result of the block" do
             @double.should_receive(:foo).with('bar') { 'baz' }
-            @double.foo('bar').should eq('baz')
+            expect(@double.foo('bar')).to eq('baz')
           end
         end
 
@@ -705,13 +786,13 @@ module RSpec
           it "fails" do
             @double.should_receive(:foo).with('bar') { 'baz' }
             expect do
-              @double.foo('wrong').should eq('baz')
+              expect(@double.foo('wrong')).to eq('baz')
             end.to raise_error(/received :foo with unexpected arguments/)
-            @double.rspec_reset
+            reset @double
           end
         end
       end
     end
-
   end
 end
+
