@@ -2,6 +2,19 @@ require 'spec_helper'
 
 module RSpec
   module Mocks
+    describe "a double stubbed with multiple return values" do
+      let(:a_double) { double }
+
+      before do
+        a_double.stub(:foo).and_return(:val_1, nil)
+      end
+
+      it 'can still set a message expectation with a single return value' do
+        a_double.should_receive(:foo).once.and_return(:val_1)
+        expect(a_double.foo).to eq(:val_1)
+      end
+    end
+
     describe "a message expectation with multiple return values and no specified count" do
       before(:each) do
         @double = double
@@ -10,24 +23,24 @@ module RSpec
       end
 
       it "returns values in order" do
-        @double.do_something.should eq @return_values[0]
-        @double.do_something.should eq @return_values[1]
-        @double.do_something.should eq @return_values[2]
-        @double.rspec_verify
+        expect(@double.do_something).to eq @return_values[0]
+        expect(@double.do_something).to eq @return_values[1]
+        expect(@double.do_something).to eq @return_values[2]
+        verify @double
       end
 
       it "falls back to a previously stubbed value" do
         @double.stub :do_something => :stub_result
-        @double.do_something.should eq @return_values[0]
-        @double.do_something.should eq @return_values[1]
-        @double.do_something.should eq @return_values[2]
-        @double.do_something.should eq :stub_result
+        expect(@double.do_something).to eq @return_values[0]
+        expect(@double.do_something).to eq @return_values[1]
+        expect(@double.do_something).to eq @return_values[2]
+        expect(@double.do_something).to eq :stub_result
       end
 
       it "fails when there are too few calls (if there is no stub)" do
         @double.do_something
         @double.do_something
-        expect { @double.rspec_verify }.to raise_error
+        expect { verify @double }.to raise_error
       end
 
       it "fails when there are too many calls (if there is no stub)" do
@@ -35,7 +48,7 @@ module RSpec
         @double.do_something
         @double.do_something
         @double.do_something
-        expect { @double.rspec_verify }.to raise_error
+        expect { verify @double }.to raise_error
       end
     end
 
@@ -47,10 +60,10 @@ module RSpec
       end
 
       it "returns values in order to consecutive calls" do
-        @double.do_something.should eq @return_values[0]
-        @double.do_something.should eq @return_values[1]
-        @double.do_something.should eq @return_values[2]
-        @double.rspec_verify
+        expect(@double.do_something).to eq @return_values[0]
+        expect(@double.do_something).to eq @return_values[1]
+        expect(@double.do_something).to eq @return_values[2]
+        verify @double
       end
     end
 
@@ -61,30 +74,30 @@ module RSpec
       end
 
       it "uses the last return value for subsequent calls" do
-        @double.do_something.should equal(11)
-        @double.do_something.should equal(22)
-        @double.do_something.should equal(22)
-        @double.rspec_verify
+        expect(@double.do_something).to equal(11)
+        expect(@double.do_something).to equal(22)
+        expect(@double.do_something).to equal(22)
+        verify @double
       end
 
       it "fails when called less than the specified number" do
-        @double.do_something.should equal(11)
-        expect { @double.rspec_verify }.to raise_error(RSpec::Mocks::MockExpectationError)
+        expect(@double.do_something).to equal(11)
+        expect { verify @double }.to raise_error(RSpec::Mocks::MockExpectationError)
       end
 
       context "when method is stubbed too" do
         before { @double.stub(:do_something).and_return :stub_result }
 
-        it "uses the stub return value for subsequent calls" do
-          @double.do_something.should equal(11)
-          @double.do_something.should equal(22)
-          @double.do_something.should equal(:stub_result)
-          @double.rspec_verify
+        it "uses the last value for subsequent calls" do
+          expect(@double.do_something).to equal(11)
+          expect(@double.do_something).to equal(22)
+          expect(@double.do_something).to equal(22)
+          verify @double
         end
 
         it "fails when called less than the specified number" do
-          @double.do_something.should equal(11)
-          expect { @double.rspec_verify }.to raise_error(RSpec::Mocks::MockExpectationError)
+          expect(@double.do_something).to equal(11)
+          expect { verify @double }.to raise_error(RSpec::Mocks::MockExpectationError)
         end
       end
     end
@@ -96,16 +109,16 @@ module RSpec
       end
 
       it "uses the last return value for subsequent calls" do
-        @double.do_something.should equal(11)
-        @double.do_something.should equal(22)
-        @double.do_something.should equal(22)
-        @double.rspec_verify
+        expect(@double.do_something).to equal(11)
+        expect(@double.do_something).to equal(22)
+        expect(@double.do_something).to equal(22)
+        verify @double
       end
 
       it "fails when called less than the specified number" do
         @double.do_something
         @double.do_something
-        expect { @double.rspec_verify }.to raise_error
+        expect { verify @double }.to raise_error
       end
 
       it "fails fast when called greater than the specified number" do
