@@ -7,19 +7,19 @@ module RSpec
         @mock = double("mock").as_null_object
       end
       it "answers false for received_message? when no messages received" do
-        @mock.received_message?(:message).should be_false
+        expect(@mock.received_message?(:message)).to be_false
       end
       it "answers true for received_message? when message received" do
         @mock.message
-        @mock.received_message?(:message).should be_true
+        expect(@mock.received_message?(:message)).to be_true
       end
       it "answers true for received_message? when message received with correct args" do
         @mock.message 1,2,3
-        @mock.received_message?(:message, 1,2,3).should be_true
+        expect(@mock.received_message?(:message, 1,2,3)).to be_true
       end
       it "answers false for received_message? when message received with incorrect args" do
         @mock.message 1,2,3
-        @mock.received_message?(:message, 1,2).should be_false
+        expect(@mock.received_message?(:message, 1,2)).to be_false
       end
     end
   end
