@@ -2,22 +2,10 @@ require 'spec_helper'
 
 module RSpec
   module Mocks
-    describe Serialization do
+    describe "Serialization of mocked objects" do
 
       class SerializableObject < Struct.new(:foo, :bar); end
 
-      class SerializableMockProxy
-        attr_reader :mock_proxy
-
-        def initialize(mock_proxy)
-          @mock_proxy = mock_proxy
-        end
-
-        def ==(other)
-          other.class == self.class && other.mock_proxy == mock_proxy
-        end
-      end
-
       def self.with_yaml_loaded(&block)
         context 'with YAML loaded' do
           module_eval(&block)
@@ -28,7 +16,7 @@ module RSpec
         context 'without YAML loaded' do
           before do
             # We can't really unload yaml, but we can fake it here...
-            @orig_yaml_constant = Object.send(:remove_const, :YAML)
+            hide_const("YAML")
             Struct.class_eval do
               alias __old_to_yaml to_yaml
               undef to_yaml
@@ -38,7 +26,6 @@ module RSpec
           module_eval(&block)
 
           after do
-            Object.const_set(:YAML, @orig_yaml_constant)
             Struct.class_eval do
               alias to_yaml __old_to_yaml
               undef __old_to_yaml
@@ -59,7 +46,7 @@ module RSpec
         end
 
         it 'serializes to yaml the same with and without stubbing, using YAML.dump' do
-          expect { set_stub }.to_not change { YAML.dump(serializable_object) }
+          expect { set_stub }.to_not change { ::YAML.dump(serializable_object) }
         end
       end
 
@@ -73,12 +60,12 @@ module RSpec
 
         if compiled_with_psych
           context 'using Syck as the YAML engine' do
-            before(:each) { YAML::ENGINE.yamler = 'syck' }
+            before(:each) { ::YAML::ENGINE.yamler = 'syck' }
             it_behaves_like 'normal YAML serialization'
           end
 
           context 'using Psych as the YAML engine' do
-            before(:each) { YAML::ENGINE.yamler = 'psych' }
+            before(:each) { ::YAML::ENGINE.yamler = 'psych' }
             it_behaves_like 'normal YAML serialization'
           end
         else
@@ -88,24 +75,15 @@ module RSpec
 
       without_yaml_loaded do
         it 'does not add #to_yaml to the stubbed object' do
-          serializable_object.should_not respond_to(:to_yaml)
+          expect(serializable_object).not_to respond_to(:to_yaml)
           set_stub
-          serializable_object.should_not respond_to(:to_yaml)
+          expect(serializable_object).not_to respond_to(:to_yaml)
         end
       end
 
       it 'marshals the same with and without stubbing' do
         expect { set_stub }.to_not change { Marshal.dump(serializable_object) }
       end
-
-      describe "an object that has its own mock_proxy instance variable" do
-        let(:serializable_object) { RSpec::Mocks::SerializableMockProxy.new(:my_mock_proxy) }
-
-        it 'does not interfere with its marshalling' do
-          marshalled_copy = Marshal.load(Marshal.dump(serializable_object))
-          marshalled_copy.should eq serializable_object
-        end
-      end
     end
   end
 end
